import io.github.jaredmdobson.concentus.*
import kotlinx.coroutines.*
import kotlinx.serialization.json.Json
import kotlinx.serialization.json.encodeToJsonElement
import java.io.File
import kotlin.concurrent.fixedRateTimer

/**
 * Created by LoliBall on 2024/8/31 5:26.
 * https://github.com/WhichWho
 */

var SERVER_HOST = "127.0.0.1"
var SERVER_PORT = 7860
var ECHO_CANCELLATION = false // 回声消除

val RAW_PCM_PASS = false // 不进行opus压缩pcm直传

// 2.5, 5,   10,  20,  40,   60 ms
// 120, 240, 480, 960, 1920, 2880
val PACKET_SAMPLES = 2880 // 48000Hz * 60ms = 2880
val SAMPLE_RATE = 48000 // 8000、12000、16000、24000、48000
val CHANNELS = 1
val BUFFER_SIZE = PACKET_SAMPLES * CHANNELS * 2 // * 2是short | 48000Hz * 2(short) * 60ms = 5760B
val AUDIO_FORMAT = AudioConvert.getAudioFormat(SAMPLE_RATE, CHANNELS)

val OPUS_BITRATE = OpusConstants.OPUS_AUTO // 500 - 300000
val OPUS_COMPLEXITY = 10 // 0: 快,质量差 10: 慢,质量好
val OPUS_SIGNAL_TYPE = OpusSignal.OPUS_SIGNAL_VOICE
val OPUS_MODE = OpusMode.MODE_HYBRID

fun main(args: Array<String>) {
    val scope = CoroutineScope(Dispatchers.IO)
    val conf = Json.decodeFromString<Config>(File("vc.conf").readText()).also {
        SERVER_HOST = it.SERVER_HOST
        SERVER_PORT = it.SERVER_PORT
        ECHO_CANCELLATION = it.ECHO_CANCELLATION
    }
    val jobServer = if (conf.SERVER) {
        scope.launch {
            Server.main(emptyArray())
        }
    } else null
    val jobClient = if (conf.CLIENT) {
        scope.launch {
            if (conf.SERVER) delay(1000) // 等服务器起来
            Client.main(emptyArray())
        }
    } else null
    if (conf.STATISTICS) {
        fixedRateTimer(period = 1000) {
            "compress rate: %.2f%%".format(compressRateOpus.get() * 100f / compressRatePcm.get()).loge()
            compressRateOpus.set(0)
            compressRatePcm.set(0)
        }
    }
    runBlocking {
        jobServer?.join()
        jobClient?.join()
    }
}
